﻿using System;
using System.IO;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using System.Diagnostics;
using System.Drawing;
using System.Threading;

namespace SnapToLayer
{
    public partial class TestForm : Form
    {
        public TestForm()
        {
            InitializeComponent();
        }

        private void TestForm_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.CurrentExtent = new RectangleShape(-97.7591, 30.3126, -97.7317, 30.2964);
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 198, 255, 255));

            //Displays the World Map Kit as a background.
            ThinkGeo.MapSuite.DesktopEdition.WorldMapKitWmsDesktopOverlay worldMapKitDesktopOverlay = new ThinkGeo.MapSuite.DesktopEdition.WorldMapKitWmsDesktopOverlay();
            winformsMap1.Overlays.Add(worldMapKitDesktopOverlay);

            string fileName1 = @"..\..\data\polygon.txt";
            StreamReader sr1 = new StreamReader(fileName1);

            string fileName2 = @"..\..\data\line.txt";
            StreamReader sr2 = new StreamReader(fileName2);

            //SnapToLayerEditInteractiveOverlay to snap dragged control point to nearest vertex of layer if within tolerance.
            SnapToLayerEditInteractiveOverlay snapToLayerEditInteractiveOverlay = new SnapToLayerEditInteractiveOverlay();
            snapToLayerEditInteractiveOverlay.Snaped += new EventHandler<MapMouseMoveInteractiveOverlayEventArgs>(snapToLayerEditInteractiveOverlay_Snaped);
            snapToLayerEditInteractiveOverlay.UnSnaped += new EventHandler<EventArgs>(snapToLayerEditInteractiveOverlay_UnSnaped);

            //inMemoryFeatureLayer used to be snapped to.
            InMemoryFeatureLayer polygonInMemoryFeatureLayer = new InMemoryFeatureLayer();
            polygonInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Park1;
            polygonInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            polygonInMemoryFeatureLayer.InternalFeatures.Add("Polygon", new Feature(BaseShape.CreateShapeFromWellKnownData(sr1.ReadLine())));

            InMemoryFeatureLayer pointInMemoryFeatureLayer = new InMemoryFeatureLayer();
            pointInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.Capital1;
            pointInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            InMemoryFeatureLayer hightlightLayer = new InMemoryFeatureLayer();
            hightlightLayer.FeatureSource.Open();
            hightlightLayer.Columns.Add(new FeatureSourceColumn("Label"));
            hightlightLayer.FeatureSource.Close();
            hightlightLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(PointSymbolType.Circle,new GeoSolidBrush(GeoColor.StandardColors.Red),new GeoPen(GeoColor.StandardColors.Black),15);
            hightlightLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = new TextStyle("Label", new GeoFont("Arial", 9.0f), new GeoSolidBrush(GeoColor.StandardColors.Red)) { XOffsetInPixel = 10, YOffsetInPixel = 10 };
            hightlightLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay inMemoryOverlay = new LayerOverlay();
            inMemoryOverlay.Layers.Add("InMemoryFeatureLayer", polygonInMemoryFeatureLayer);
            inMemoryOverlay.Layers.Add("PointInMemoryFeatureLayer", pointInMemoryFeatureLayer);
            inMemoryOverlay.Layers.Add("hightlightLayer", hightlightLayer);
            winformsMap1.Overlays.Add("InMemoryOverlay", inMemoryOverlay);

            snapToLayerEditInteractiveOverlay.EditShapesLayer.InternalFeatures.Add("MultiLine", new Feature(BaseShape.CreateShapeFromWellKnownData(sr2.ReadLine())));

            //Sets the PointStyle for the non dragged control points.
            snapToLayerEditInteractiveOverlay.ControlPointStyle = new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColor.StandardColors.PaleGoldenrod), new GeoPen(GeoColor.StandardColors.Black), 8);
            //Sets the PointStyle for the dragged control points.
            snapToLayerEditInteractiveOverlay.DraggedControlPointStyle = new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColor.StandardColors.Red), new GeoPen(GeoColor.StandardColors.Orange, 2), 10);

            snapToLayerEditInteractiveOverlay.ToSnapInMemoryFeatureLayer = polygonInMemoryFeatureLayer;

            //Example using Screen (Pixel) coordinates for tolerance.
            snapToLayerEditInteractiveOverlay.ToleranceType = ToleranceCoordinates.Screen;
            snapToLayerEditInteractiveOverlay.Tolerance = 25;

            snapToLayerEditInteractiveOverlay.CalculateAllControlPoints();
            winformsMap1.EditOverlay = snapToLayerEditInteractiveOverlay;
            winformsMap1.Refresh();
        }

        private void snapToLayerEditInteractiveOverlay_UnSnaped(object sender, EventArgs e)
        {
            LayerOverlay inMemoryOverlay = (LayerOverlay)winformsMap1.Overlays["InMemoryOverlay"];
            InMemoryFeatureLayer hightlightLayer = (InMemoryFeatureLayer)inMemoryOverlay.Layers["hightlightLayer"];
            hightlightLayer.InternalFeatures.Clear();
            winformsMap1.Refresh();
        }

        private void snapToLayerEditInteractiveOverlay_Snaped(object sender, MapMouseMoveInteractiveOverlayEventArgs e)
        {
            Point point = new Point((int)e.InteractionArguments.ScreenX, (int)e.InteractionArguments.ScreenY);
            Cursor.Position = winformsMap1.PointToScreen(point);

            LayerOverlay inMemoryOverlay = (LayerOverlay)winformsMap1.Overlays["InMemoryOverlay"];
            InMemoryFeatureLayer hightlightLayer = (InMemoryFeatureLayer)inMemoryOverlay.Layers["hightlightLayer"];
            PointShape pointShape = ExtentHelper.ToWorldCoordinate(e.InteractionArguments.CurrentExtent, new ScreenPointF(point.X, point.Y), e.InteractionArguments.MapWidth, e.InteractionArguments.MapHeight);
            Feature feature = pointShape.GetFeature();
            feature.ColumnValues["Label"] = string.Format("Longitude:{0:0.00} - Latitude:{1:0.00}", pointShape.X, pointShape.Y);
            hightlightLayer.InternalFeatures.Clear();
            hightlightLayer.InternalFeatures.Add(feature);
            winformsMap1.Refresh();
        }

        private void winformsMap1_MouseMove(object sender, MouseEventArgs e)
        {
            //Displays the X and Y in screen coordinates.
            statusStrip1.Items["toolStripStatusLabelScreen"].Text = "X:" + e.X + " Y:" + e.Y;

            //Gets the PointShape in world coordinates from screen coordinates.
            PointShape pointShape = ExtentHelper.ToWorldCoordinate(winformsMap1.CurrentExtent, new ScreenPointF(e.X, e.Y), winformsMap1.Width, winformsMap1.Height);

            //Displays world coordinates.
            statusStrip1.Items["toolStripStatusLabelWorld"].Text = "(world) X:" + Math.Round(pointShape.X, 4) + " Y:" + Math.Round(pointShape.Y, 4);
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
